/*
 * Frame level networking
 *
 * Copyright (C) 2003 Juha Aatrokoski, Timo Lilja,
 *   Leena Salmela, Teemu Takanen, Aleksi Virtanen.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: network.h,v 1.3 2003/05/12 20:04:09 ttakanen Exp $
 *
 */

#ifndef NET_NETWORK_H
#define NET_NETWORK_H

#include "lib/types.h"
#include "drivers/gnd.h"

void network_init(void);

network_address_t network_get_source_address(int interface);
network_address_t network_get_broadcast_address(void);
network_address_t network_get_loopback_address(void);
int network_get_mtu(network_address_t local_address);

int network_send(network_address_t source,
		 network_address_t destination,
		 uint32_t protocol_id,
		 int length,
		 void *buffer);

void network_free_frame(void *frame);

/* Return values of the network frame layer functions. */
#define NET_OK 0
#define NET_ERROR -1
#define NET_DOESNT_EXIST -2

#define NETWORK_BROADCAST_ADDRESS 0xffffffff
#define NETWORK_LOOPBACK_ADDRESS  0x00000000
#define NETWORK_MAX_MTU 4096

#endif /* NET_NETWORK_H */



